#!/usr/bin/env cwl-runner
cwlVersion: v1.2
class: Workflow
requirements:
  StepInputExpressionRequirement: {}
  InlineJavascriptRequirement: {}
  MultipleInputFeatureRequirement: {}
  SubworkflowFeatureRequirement: {}
  ScatterFeatureRequirement: {}

label: Metagenomics workflow
doc: |
    Workflow for Metagenomics from raw reads to annotated bins.
    Steps:
      - workflow_quality.cwl:
          - FastQC (control)
          - fastp (trimming)
      - Kraken2 (Taxonomic Read Classification
      - SPAdes (Assembly)
      - QUAST (Assembly quality report)
      - BBmap (Read mapping to assembly)
      - sam_to_bam (sam to indexed bam)
      - metabatContigDepths (jgi_summarize_bam_contig_depths)
      - MetaBat2 (binning)

outputs:
  fastqc_output:
    label: FASTQC
    doc: Quality reporting by FASTQC
    type: Directory[]
    outputSource: workflow_quality/files_to_folder_fastqc
  filter_output:
    label: Filtered reads
    doc: Reads filtered output folder
    type: Directory[]
    outputSource: workflow_quality/files_to_folder_filtered
  kraken2_output:
    label: KRAKEN2
    doc: Classification output folder by kraken
    type: Directory
    outputSource: kraken_files_to_folder/results
  spades_output:
    label: SPADES
    doc: Metagenome assembly output by SPADES
    type: Directory
    outputSource: spades_files_to_folder/results
  quast_output:
    label: QUAST
    doc: Quast analysis output folder
    type: Directory
    outputSource: quast_files_to_folder/results
  bam_output:
    label: BAM files
    doc: Mapping results in indexed BAM format
    type: Directory
    outputSource: sorted_bam_files_to_folder/results
  metabat2_output:
    label: MetaBat2
    doc: MetaBat2 output directory
    type: Directory
    outputSource: metabat_files_to_folder/results
  # sapp_output:
  #   label: SAPP Genomes
  #   type: Directory
  #   doc: Folder containing genomes annotated by SAPP 
  #   outputSource: sapp_files_to_folder/results

inputs:
  identifier:
    type: string
    doc: Identifier for this dataset used in this workflow
    label: identifier used
  filter_rrna:
    type: boolean
    doc: rRNA read filtering using ...
    label: rRNA filtering
    default: false
  forward_reads:
    type: File[]
    doc: forward sequence file locally
    label: forward reads
  reverse_reads:
    type: File[]
    doc: reverse sequence file locally
    label: reverse reads
  threads:
    type: int?
    doc: number of threads to use for computational processes
    label: number of threads
    default: 2
  memory:
    type: int?
    doc: maximum memory usage in megabytes
    label: memory usage (mb)
    default: 4000
  gzip:
    type:
    - "null"
    - boolean
    doc: input data is gzip compressed
    label: gzip compression
    default: false
  kraken_database:
    type: string
    doc: database location of kraken2
    label: Kraken2 database
  pacbio_reads:
    type: File[]?
    doc: file with PacBio reads locally
    label: pacbio reads

steps:
  #########################################
  # Workflow for quality and filtering using fastqc, fastp and optionally bbduk
  workflow_quality:
    label: Quality and filtering workflow
    doc: Quality assessment of illumina reads with rRNA filtering option
    run: workflow_quality.cwl
    scatter: [forward_reads, reverse_reads]
    scatterMethod: dotproduct
    in:
      filter_rrna: filter_rrna
      forward_reads: forward_reads
      reverse_reads: reverse_reads
      memory: memory
      threads: threads
      identifier: identifier
      step:
        default: 1
    out: [QC_reverse_reads, QC_forward_reads, files_to_folder_filtered, files_to_folder_fastqc]
  #########################################
  # Workflow for the kraken run
  workflow_kraken:
    label: Kraken2 workflow
    doc: Read classification using the kraken2 database
    run: ../metagenomics/kraken2/kraken2.cwl
    scatter: [forward_reads, reverse_reads]
    scatterMethod: dotproduct
    in:
      identifier: identifier
      threads: threads
      database: kraken_database
      forward_reads: workflow_quality/QC_forward_reads
      reverse_reads: workflow_quality/QC_reverse_reads
      gzip: gzip # Based on regex?
    out: [kraken, report]
  #########################################
  # assembly using SPADES
  workflow_spades:
    doc: Genome assembly using spades with illumina/pacbio reads
    label: SPADES assembly
    run: ../assembly/spades.cwl
    in:
      forward_reads: workflow_quality/QC_forward_reads
      reverse_reads: workflow_quality/QC_reverse_reads
      pacbio_reads: pacbio_reads
      metagenomics:
        default: true
      memory: memory
      threads: threads
    out: [contigs, scaffolds, assembly_graph, contigs_assembly_graph, scaffolds_assembly_graph, contigs_before_rr, params, log, internal_config, internal_dataset]
  #########################################
  # assembly quality assessment using quast
  workflow_quast:
    doc: Genome assembly quality assessment using Quast
    label: Quast workflow
    run: ../quast/quast.cwl
    in:
      assembly: workflow_spades/scaffolds
    out: [basicStats, icarusDir, icarusHtml, quastReport, quastLog, transposedReport]
  #########################################
  # read mapping using bbmap
  workflow_bbmap:
    label: bbmap read mapping
    doc: illumina read mapping using BBmap
    run: ../bbmap/bbmap.cwl
    scatter: [forward_reads, reverse_reads]
    scatterMethod: dotproduct
    in:
      identifier: identifier
      reference: workflow_spades/scaffolds
      forward_reads: workflow_quality/QC_forward_reads
      reverse_reads: workflow_quality/QC_reverse_reads
      memory: memory
    out: [sam, stats, covstats, rpkm]
  #########################################
  # read mapping using bbmap
  workflow_sam_to_sorted_bam:
    label: sam conversion to sorted bam
    doc: sam file conversion to a sorted indexed bam file
    scatter: [sam]
    scatterMethod: dotproduct
    run: ../samtools/sam_to_sorted-bam.cwl
    in:
      identifier: identifier
      sam: workflow_bbmap/sam
      threads: threads
    out: [sortedbam]
  #########################################
  # Merge bam files
  workflow_merge_bam_files:
    label: Merging bam files
    doc: Merge multiple bam files obtained from the sam to bam conversion module
    run: ../samtools/merge-sorted-bam.cwl
    in:
      identifier: identifier
      bam: workflow_sam_to_sorted_bam/sortedbam
      threads: threads
    out: [sortedbam]
  #########################################
  # depth file retrieval for metabat2
  workflow_metabat_contig_depths:
    label: depth file from metabat2
    doc: Execution of metabat2 to obtain the depth file used in the binning process
    run: ../metagenomics/metabat2/metabatContigDepths.cwl
    in:
      bamFile: workflow_merge_bam_files/sortedbam
    out: [depths]
  #########################################
  # assembly binning using metabat2
  workflow_metabat:
    doc: Binning procedure using metabat2
    label: binning process
    run: ../metagenomics/metabat2/metabat2.cwl
    in:
      identifier: identifier
      threads: threads
      assembly: workflow_spades/scaffolds
      depths: workflow_metabat_contig_depths/depths
    out: [bins, log]
  #########################################
  # bin annotation using SAPP
  # workflow_sapp:
  #   doc: Semantic Annotation Platform with Provenance to annotate each bin obtained from metabat2
  #   label: Semantic Annotation Platform with Provenance
  #   run: workflow_sapp.cwl
  #   scatter: [fasta]
  #   in:
  #     fasta: workflow_metabat/bins
  #     codon:
  #       valueFrom: ${ return 11; }
  #     identifier:
  #       valueFrom: ${ return inputs.fasta.nameroot; }
  #     single:
  #       default: true
  #     threads: threads
  #   out: [conversion, prodigal, interproscan]
#############################################
#### Move to folder if not part of a workflow
  kraken_files_to_folder:
    doc: Preparation of kraken output files to a specific output folder
    label: Kraken output
    run: ../expressions/files_to_folder.cwl
    in:
      files: 
        source: [workflow_kraken/kraken, workflow_kraken/report]
        linkMerge: merge_flattened
        pickValue: all_non_null
      destination: 
        valueFrom: $("Step_3_Kraken")
    out:
      [results]
#############################################
  spades_files_to_folder:
    doc: Preparation of spades output files to a specific output folder
    label: SPADES output
    run: ../expressions/files_to_folder.cwl
    in:
      files: #workflow_spades/json,
        source: [workflow_spades/contigs, workflow_spades/scaffolds, workflow_spades/assembly_graph, workflow_spades/contigs_assembly_graph, workflow_spades/scaffolds_assembly_graph, workflow_spades/contigs_before_rr, workflow_spades/params, workflow_spades/log, workflow_spades/internal_config, workflow_spades/internal_dataset]
      destination: 
        valueFrom: $("Step_4_Assembly")
    out:
      [results]
#############################################
  quast_files_to_folder:
    doc: Preparation of quast output files to a specific output folder
    label: QUAST output
    run: ../expressions/files_to_folder.cwl
    in:
      identifier: identifier
      files: 
        source: [workflow_quast/quastLog, workflow_quast/icarusHtml, workflow_quast/quastReport, workflow_quast/transposedReport]
        linkMerge: merge_flattened
      folders:
        source: [workflow_quast/basicStats, workflow_quast/icarusDir]
        linkMerge: merge_flattened
      destination: 
        valueFrom: $("Step_5_Quast_")$(inputs.identifier)
    out:
      [results]
#############################################
  sorted_bam_files_to_folder:
    doc: Preparation of bam files output to a specific output folder
    label: BAM output
    run: ../expressions/files_to_folder.cwl
    in:
      files: 
        source: [workflow_sam_to_sorted_bam/sortedbam, workflow_bbmap/stats, workflow_bbmap/covstats, workflow_bbmap/rpkm]
        linkMerge: merge_flattened
      destination: 
        valueFrom: $("Step_6_BBMAP_BAM")
    out:
      [results]
  metabat_files_to_folder:
    doc: Preparation of metabatt output files to a specific output folder
    label: metabat output
    run: ../expressions/files_to_folder.cwl
    in:
      files: 
        source: [workflow_metabat/log, workflow_metabat/bins]
        linkMerge: merge_flattened
      destination: 
        valueFrom: $("Step_7_Metabat")
    out:
      [results]
  # sapp_files_to_folder:
  #   doc: Preparation of sapp output files to a specific output folder
  #   label: SAPP output
  #   run: ../expressions/files_to_folder.cwl
  #   in:
  #     files: 
  #       source: [workflow_sapp/conversion, workflow_sapp/prodigal , workflow_sapp/interproscan]
  #       linkMerge: merge_flattened
  #     destination: 
  #       valueFrom: $("Step_8_SAPP")
  #   out:
  #     [results]
#############################################

s:author:
  - class: s:Person
    s:identifier: https://orcid.org/0000-0001-8172-8981
    s:email: mailto:jasper.koehorst@wur.nl
    s:name: Jasper Koehorst
  - class: s:Person
    s:identifier: https://orcid.org/0000-0001-9524-5964
    s:email: mailto:bart.nijsse@wur.nl
    s:name: Bart Nijsse

s:citation: https://m-unlock.nl
s:codeRepository: https://gitlab.com/m-unlock/cwl
s:dateCreated: "2020-00-00"
s:license: https://spdx.org/licenses/Apache-2.0 
s:copyrightHolder: "UNLOCK - Unlocking Microbial Potential"

$namespaces:
  s: https://schema.org/
  edam: http://edamontology.org/

$schemas:
 - http://edamontology.org/EDAM_1.18.owl
